uniform vec4 diffuseColor;
uniform int hasTexture;
uniform float specularIntensity;
uniform float specularHardness;
uniform float incandescence;
uniform vec3 eyePosition;
uniform int receiveShadows;
uniform vec4 fogColor;
uniform int fogEnabled;

#if defined(PT_PCF_SHADOWS)
uniform sampler2DShadow shadowMapTexture;
#else
uniform sampler2D shadowMapTexture;
#endif

uniform vec3 u_DirLightSourceColor[1];
uniform vec3 u_DirLightSourceDirection[1];
uniform vec3 u_AmbientLightSourceColor;

uniform float fogStartDistance;
uniform float farPlane;

varying vec2 texCoord;
varying vec4 lightViewportTexCoordDivW;
varying vec3 worldPosition;
varying vec3 worldNormal;
varying float viewDepth;

vec3 computeLighting(vec3 normalVector, vec3 lightDirection, vec3 lightColor, float attenuation) {
    return lightColor * max(dot(normalVector, lightDirection), 0.0) * attenuation;
}

void main() {
    vec3 lightDirection = -normalize(u_DirLightSourceDirection[0] * 2.0); //TODO: we have two light direction variables. need only one
    float shadow = 1.0;
    float specular = 0.0;

    if(receiveShadows == 1){
        float dt = dot(lightDirection, normalize(worldNormal));
        float bias = max(0.0025 * (1.0 - dt), 0.0005);
        if (dt > 0.0) {
#ifdef PT_PCF_SHADOWS
#ifdef GL_EXT_shadow_samplers 
            shadow = shadow2DProjEXT(shadowMapTexture, lightViewportTexCoordDivW + vec4(0, 0, -bias / lightViewportTexCoordDivW.w, 0));
#else
            shadow = shadow2DProj(shadowMapTexture, lightViewportTexCoordDivW + vec4(0, 0, -bias / lightViewportTexCoordDivW.w, 0)).z;
#endif
#else
            float depth = texture2DProj(shadowMapTexture, lightViewportTexCoordDivW + vec4(0, 0, -bias / lightViewportTexCoordDivW.w, 0)).x;
            float R = (lightViewportTexCoordDivW + vec4(0, 0, -bias / lightViewportTexCoordDivW.w, 0)).p / (lightViewportTexCoordDivW + vec4(0, 0, -bias / lightViewportTexCoordDivW.w, 0)).q;
            
            shadow = (R <= depth) ? 1.0 : 0.0;
#endif
        }
    }
    if (specularIntensity > 0.0 && specularHardness > 0.0) {
        float normFactor = (specularHardness + 2.0) / 2.0;
        vec3 reflection = reflect(-lightDirection, normalize(worldNormal));

        specular = normFactor * pow(max(dot(reflection, normalize(eyePosition - worldPosition)), 0.0), specularHardness);
        specular = clamp(specular, 0.0, 1.0) * specularIntensity;
    }
    
    float inc = mix(0.5, 0.0, incandescence);
    vec3 light = computeLighting(normalize(worldNormal), lightDirection, u_DirLightSourceColor[0], 1.0);
    vec4 shade = vec4(light, 1) * (vec4(shadow, shadow, shadow, 1.0) + vec4(0.5, 0.5, 0.5, 0.0));
    
    if (hasTexture == 1) {
        gl_FragColor = texture2D(CC_Texture0, texCoord) * diffuseColor * mix(vec4(1.0, 1.0, 1.0, 0.0), shade, vec4(inc, inc, inc, 1.0)); //default was 0.5//todo: need add color handler here
    }
    else {
        gl_FragColor = diffuseColor * mix(vec4(1.0, 1.0, 1.0, 0.0), shade, vec4(inc, inc, inc, 1.0)); //default was 0.5//todo: need add color handler here
    }
    
    //Specular
    if (shadow == 1.0 && specular > 0.0) {
        gl_FragColor += vec4(specular, specular, specular, 1.0);
    }

    //Fog
    if (fogEnabled == 1 && viewDepth > fogStartDistance) {
        gl_FragColor = mix(gl_FragColor, fogColor, (viewDepth - fogStartDistance) / (farPlane - fogStartDistance));
    }
    
    gl_FragColor.a = 1.0;
}
